function PD = Multirings(parameters, CreateDefaultFlag, app)

% Print a double rings structures
% The rings are orthogonal to each other
%
% -------------------------------------------------------------------------
%
%       * * *       
%   *     *     *   
% *       *       * 
% *       *       * 
% *       *       * 
%   *     *     *
%       * * *   
%         *  
%         *  <--- bottom pillar
%         *  
% --------------------------- Parameters ----------------------------------



if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag

    Variable    ={'r1';'r2';'p1';'p4';'pitch';'pr'};           % name of each variable used for this scruture
    Description ={'Outer ring radius';'Inner ring radius';'Bottom pillar height';'Top pillar Height';'Voxel pitch';'Pressure'};       % description of each variable
    Value       ={10;6;5;1;0.5;60};                      % default values 
    Unit        ={'um';'um';'um';'um';'um';'mbar'};                 % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR 
        temp=parameters{(parameters.Row{k}),'Value'};                   % take the value for each variable
        if ischar(temp{:})                                              % convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                                % create the obj. structure based on the name of each variable
        temp=[];
    end
    
    PD=MakeMultirings(obj);                                             % call the function building the structure's voxels cloud

end
end

function PD = MakeMultirings(obj)
% function that creats multiple rings (up to 3)
% all orthogonal to one another


xstart = 0;
ystart = 0;

StructureHeight = obj.p1 + 2*obj.r1;

obj.pillar2 = round(obj.r1-obj.r2);

Circumference1 = 2*pi*obj.r1;
Circumference2 = 2*pi*obj.r2;
NoVoxels1 = round(Circumference1/obj.pitch);
NoVoxels2 = round(Circumference2/obj.pitch);
layer = 0;

xpillar1 = [];         ypillar1 = [];         zpillar1 = [];

xpillarstart = obj.r1;
ypillarstart = obj.r2;

if obj.p1 > 0
       
    for i=obj.pitch:obj.pitch:(obj.p1-obj.pitch)
        
        xpillar1 = [xpillar1 xpillarstart];
        ypillar1 = [ypillar1 ypillarstart];
        zpillar1 = [zpillar1 i];
        
    end
    
end

xpillar2 = [];
ypillar2 = [];
zpillar2 = [];

if obj.r1 > obj.r2

    for i=(obj.p1+obj.pitch):obj.pitch:(obj.p1+obj.pillar2)

        xpillar2 = [xpillar2 xpillarstart];
        ypillar2 = [ypillar2 ypillarstart];
        zpillar2 = [zpillar2 i];

    end

end


xpillar3 = [];  ypillar3 = [];  zpillar3 = [];
startzpillar3= obj.p1 + obj.pillar2 + 2*obj.r2;

if obj.r1 > obj.r2

    for i=startzpillar3:obj.pitch:StructureHeight-obj.pitch

        xpillar3 = [xpillar3 xpillarstart];
        ypillar3 = [ypillar3 ypillarstart];
        zpillar3 = [zpillar3 i];

    end

end

xpillar4 = [];  ypillar4 = [];  zpillar4 = [];
startzpillar4= obj.p1 + obj.pillar2 + 2*obj.r2;

for i=StructureHeight:obj.pitch:(StructureHeight+obj.p4)
    
    xpillar4 = [xpillar4 xpillarstart];
    ypillar4 = [ypillar4 ypillarstart];
    zpillar4 = [zpillar4 i];
    
end


PiePart1 = (2*pi/NoVoxels1); % part of the pie in radians

ring1_left_rad = pi:PiePart1:(2*pi);
ring1_right_rad = (pi+PiePart1):-PiePart1:0;

ring1_left_x = obj.r1*sin(ring1_left_rad) + xpillarstart;
ring1_left_z = obj.r1*cos(ring1_left_rad) + obj.p1 + obj.r1;
ring1_left_y = ypillarstart * ones(1,length(ring1_left_z));

ring1_right_x = obj.r1*sin(ring1_right_rad) + xpillarstart;
ring1_right_z = obj.r1*cos(-ring1_right_rad) + obj.p1 + obj.r1;
ring1_right_y = ypillarstart * ones(1,length(ring1_right_z));

PiePart2 = (2*pi/NoVoxels2); % part of the pie in radians

ring2_left_rad = pi:PiePart2:(2*pi);
ring2_right_rad = (pi+PiePart2):-PiePart2:0;

ring2_left_z = obj.r2*cos(ring2_left_rad) + obj.p1 + obj.pillar2 + obj.r2;
ring2_left_x = xpillarstart * ones(1,length(ring2_left_z));
ring2_left_y = obj.r2*sin(ring2_left_rad) + ypillarstart;

ring2_right_z = obj.r2*cos(-ring2_right_rad) + obj.p1 + obj.pillar2 + obj.r2;
ring2_right_x = xpillarstart * ones(1,length(ring2_right_z));
ring2_right_y = obj.r2*sin(ring2_right_rad) + ypillarstart;



x = [xpillar1 ring1_left_x ring1_right_x xpillar2 ring2_left_x ring2_right_x xpillar3 xpillar4];
y = [ypillar1 ring1_left_y ring1_right_y ypillar2 ring2_left_y ring2_right_y ypillar3 ypillar4];
z = [zpillar1 ring1_left_z ring1_right_z zpillar2 ring2_left_z ring2_right_z zpillar3 zpillar4];

%sort by z
AllData = [z; x; y]; % z x y

AllDataSortedTemp = sortrows(AllData',1); % sort only on z

% get rid of double points:
AllDataSortedTemp = unique(AllDataSortedTemp,'rows','stable');

z = AllDataSortedTemp(:,1)';
x = AllDataSortedTemp(:,2)';
y = AllDataSortedTemp(:,3)';

% GENERATE Print Data
PD.x = x';
PD.y = y';
PD.z = z';
PD.pr = obj.pr*ones(length(PD.z),1);
end