function PD = Needle(parameters, CreateDefaultFlag, app)
% Print a needle (hollow)
%
% -------------------------------------------------------------------------
%      /|     /| = tip angle
%     / |    /-| 
%    /  |
%    |  |
%    |  | 
%   --------
%
% --------------------------- Parameters ----------------------------------


if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag

    Variable    ={'radius';'vpitch';'vh';'height';'alternating';'tipangle';'pr'};     % name of each variable used for this scruture
    Description ={'Needle radius';'Voxel pitch'; 'Voxel height';'Height of the needle';'Alternate Flag';'Bevel Angle to vertical';'Pressure'};           % description of each variable
    Value       ={30;1;2;200;1;35;60};                                                          % default values 
    Unit        ={'um';'um';'um';'um';'logical';'deg';'mbar'};                                          % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR 
        temp=parameters{(parameters.Row{k}),'Value'};                   % take the value for each variable
        if ischar(temp{:})                                              % convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                                % create the obj. structure based on the name of each variable
        temp=[];
    end
    
    PD=MakeNeedle(obj);                                             % call the function building the structure's voxels cloud

end
end


function PD = MakeNeedle(obj)

% Generate needle

xstart = 0;
ystart = 0;
zstart=obj.vh;
zstep=obj.vh;
Circumference = 2*pi*obj.radius;
NoPillars = round(Circumference/obj.vpitch);
layer = 0; % layer counter

x = zeros(1,length(zstart:zstep:obj.height)*NoPillars);
y = x; z = x;
index = 1;

for i=zstart:zstep:obj.height  % z = height
    
    layer = layer + 1;% count layer
   % startrand = rand; % random starting postition every z step
    
    for j=1:1:NoPillars            % y = rows
        
        PiePart = (2*pi/NoPillars); % part of the pie in radians
        
%         if obj.random
%             x(index) = obj.radius*sin(startrand*PiePart+PiePart*(j-1))+obj.radius+xstart;
%             y(index) = obj.radius*cos(startrand*PiePart+PiePart*(j-1))+obj.radius+ystart;
        if obj.alternating
            if rem(layer,2) == 1 % odd
                x(index) = obj.radius*sin(PiePart*(j-(1/2)))+obj.radius+xstart;
                y(index) = obj.radius*cos(PiePart*(j-(1/2)))+obj.radius+ystart;
            else % i is even
                x(index) = obj.radius*sin(PiePart*(j-1))+obj.radius+xstart;
                y(index) = obj.radius*cos(PiePart*(j-1))+obj.radius+ystart;
            end
        else % just pillars
            x(index) = obj.radius*sin(PiePart*(j-1))+obj.radius+xstart;
            y(index) = obj.radius*cos(PiePart*(j-1))+obj.radius+ystart;
        end
        
        z(index) = i;
        index = index + 1;
        
    end
    
end

% Intersect the cylinder with a plane
alpharad = obj.tipangle * 2 * pi / 360;              % bevel angle in radians
beta = (90-obj.tipangle) * 2*pi / 360;               % plane angle with respect to x-axis
zintersect =  obj.height - (xstart + 2 * obj.radius) / tan(alpharad);  % Calculate z-axis intersection of the plane
zplane = zintersect + tan(beta) .* x;               % Enter every x value in the linear equation

RemovePnts = [];    % Array for points above plane

% Compare all z values with the z values of the plane
for i = 1:1:numel(z)
    
    if z(i) > zplane(i)

        RemovePnts = [RemovePnts i];
        
    end
    
end

% Remove all the points above the plane:
z(RemovePnts) = [];
x(RemovePnts) = [];
y(RemovePnts) = [];

% GENERATE Print Data
PD.x = x';
PD.y = y';
PD.z = z';
PD.pr = obj.pr*ones(length(PD.z),1);
end